/****************************************************************************
**
** This file is part of PLCEdit, an open-source cross-platform editor 
** for PLC source files (e.g. from Moeller, CoDeSys and Siemens).
** Copyright (C) 2005-2009  M. Rehfeldt
**
** This software uses classes of Trolltech Qt toolkit and is freeware. 
** This file may be used under the terms of the GNU General Public License 
** version 2.0 or (at your option) any later version as published by the 
** Free Software Foundation and appearing in the file LICENSE.GPL included 
** in the packaging of this file. 
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** Contact e-mail: M. Rehfeldt <info@plcedit.org>
** Program URL   : http://www.plcedit.org
**
****************************************************************************/


#include <QFile> 
#include <QProcess> 
#include <QWidget> 
#include <QMessageBox> 
#include <QDir> 
#include <unistd.h> 

#include "singleapplinux.h"


SingleAppLinux::SingleAppLinux(const QString &lockFile, int &argc, char *argv[]) //was char **argv 
   : QApplication(argc, argv) 
{ //! creting function
   
   //Thanks to: Peter Gaede
   
   
   lockFileStr = QString("%1/%2").arg(QDir::tempPath()).arg(lockFile); 
   flg_isRunning = checkForRunningInstance(); 
} 


SingleAppLinux::~SingleAppLinux() 
{ //! virtual destructor
	
  if (!flg_isRunning) 
  { 
    QFile file(lockFileStr); 
    if (file.exists()) 
      file.remove(); 
  } 
} 


bool SingleAppLinux::checkForRunningInstance() 
{ //! check if an instance is already running
	
  QFile file(lockFileStr); 

  // Check if lock file exists 
  if (file.exists()) 
  { 
    // Now we check if a previous instance are running 
    if (file.open(QIODevice::Text | QIODevice::ReadOnly)) 
    { 
      QString pid = QString(file.readLine()); 
      file.close(); 

      if (!pid.isEmpty()) 
      { 
        QProcess p; 

        p.start("ps", QStringList() << "--pid" << pid); 
        p.waitForFinished(); 

        QString data = QString(p.readAllStandardOutput()); 
        if (data.contains(pid)) 
          return(true); 
      } 
    } 
    else 
    { 
      QWidget w; 
      QMessageBox::critical(&w, tr("Lock file error"), QString(tr("Unable to open lock file for reading:\n%1")).arg(lockFileStr)); 
      return(true); 
    } 
  } 

  // We are the only one instance, now we write our pid to lock file 
  if (file.open(QIODevice::Text | QIODevice::WriteOnly)) 
  { 
    file.resize(0); 
    file.write(QString::number(getpid()).toAscii()); 
    file.close(); 
  } 
  else 
  { 
    QWidget w; 
    QMessageBox::critical(&w, tr("Lock file error"), QString(tr("Unable open lock file for writing:\n%1")).arg(lockFileStr)); 
    return(true); 
  } 

  return(false); 
} 


QString SingleAppLinux::lockFile() const 
{ //! return lock file flag
	
  return(lockFileStr); 
} 


bool SingleAppLinux::isRunning() const 
{ //! return running information
	
  return(flg_isRunning); 
}
